import java.io.IOException;
import java.net.SocketException;

import conet.configure.ConfigureExample;
import conet.util.GetIPaddress;
import conet.signaling.ConetSignal;
import conet.signaling.message.Register;
import conet.signaling.message.Unregister;
import conet.socket.Client;
import conet.socket.Server;
import conet.signaling.message.Find_request;
import conet.signaling.message.Find_response;
import conet.util.ip_pref;

public class ResolutionHandler implements NameResolutionService{
	private ConfigureExample configuration_file = null;
	private GetIPaddress tmpip = null;
//	private Server server = null;
	private int remote_port;
	private String[] local_NRS = null;
	
	public ResolutionHandler(){
		configuration_file = ConfigureExample.getInstance();
		configuration_file.init("conetRE.conf");
		tmpip = new GetIPaddress();
		remote_port =9999;
		local_NRS = configuration_file.getParams("local_NRS");
	}
	
	public ResolutionHandler(String conf_file, int rport){
		configuration_file = ConfigureExample.getInstance();
		configuration_file.init(conf_file);
		tmpip = new GetIPaddress();
		remote_port = rport;
		local_NRS = configuration_file.getParams("local_NRS");
	}

	@Override
	public boolean register(String content_name) {
		int distance = 0;
		byte pub_flag = 1;

		// probably we should check if inserted content name is well formed
		if(content_name !=null){
			Register msgRegister;
			Client client = new Client();
			try {
				msgRegister = new Register(content_name,tmpip.ip(),pub_flag,Integer.toString(distance));

				String serializedClass_to_NRS = ConetSignal.serialize(msgRegister); 
				for (int i=0; i<local_NRS.length; i++)
					client.transfer(local_NRS[i],remote_port,serializedClass_to_NRS);

			} catch (SocketException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

		return false;
	}

	@Override
	public boolean unregister(String content_name) {
		try {  
			if(content_name !=null){
				Unregister msgUnregister = new Unregister(content_name,tmpip.ip());
				String serializedClass = ConetSignal.serialize(msgUnregister); 
				Client client = new Client();                                               
				for (int i=0; i<local_NRS.length; i++){
					client.transfer(local_NRS[i],remote_port,serializedClass);
				}
			} 
		}catch(Exception e){
			e.printStackTrace();
		}
		return false;
	}

	@Override
	public synchronized String find(String content_name) {

		try {
			Find_response hostdata_resp;	

			Client client = new Client();
			Find_request msgFind_to_NRS = new Find_request(content_name,tmpip.ip());
			String serializedClass_to_NRS = ConetSignal.serialize(msgFind_to_NRS);
			for (int i=0; i<local_NRS.length; i++){
				client.transfer(local_NRS[i],remote_port,serializedClass_to_NRS);
			}            

			String MType =client.receive();

			if (MType.equals("Find_response")){                                          
				hostdata_resp = client.getHostdata_find_resp();                            
				if (hostdata_resp.values !=null){
					String[][] result = hostdata_resp.values;
					
					if(result[0].length == 1){
						//System.out.println("No hops required, the content is inside your subystem at: " + result[2][0]);
						return result[2][0];
					}
					else{
						String next_hop = new ip_pref().next_hop(tmpip.ip(),hostdata_resp.values );
						//System.out.println("The next hop is : "+next_hop);
						return next_hop;
					}
				}
				else {
					//System.out.println("Content Name not found");
					return null;
				}

			}
		}
		catch(Exception e){
			e.printStackTrace();
		}

		return null;
	}

}
