package conet.configure;


import java.io.*;
import java.net.URL;



/** Configure helps the loading and saving of configuration data.
  */
public class Configure
{
   /** String 'NONE' used as undefined value (i.e. null). */
   public static String NONE="NONE";


   /** The object that should be configured. */
   Configurable configurable;


   /** Creates a new Configure. */
   public Configure(Configurable configurable)
   {  this.configurable=configurable;
   }

       
   /** Loads configuration from the given file. */
   public void load(String file)
   {  if (file==null)
      {  return;
      }
      //else
      try
      {  readFrom(new FileReader(file));
      }
      catch (Exception e)
      {  System.err.println("WARNING: error reading file \""+file+"\"");
         return;
      }
   }


   /** Loads configuration from the given URL. */
   public void load(URL url)
   {  if (url==null)
      {  return;
      }
      //else
      try
      {  readFrom(new InputStreamReader(url.openStream()));
      }
      catch (Exception e)
      {  System.err.println("WARNING: error reading from \""+url+"\"");
         return;
      }
   }


   /** Saves configuration on the given file. */
   public void save(String file)
   {  if (file==null) return;
      //else
      try
      {  writeTo(new FileWriter(file));
      }
      catch (IOException e)
      {  System.err.println("ERROR writing on file \""+file+"\"");
      }         
   }


   /** Reads configuration from the given Reader. */
   private void readFrom(Reader rd) throws java.io.IOException
   {  BufferedReader in=new BufferedReader(rd);           
      while (true)
      {  String line=null;
         try { line=in.readLine(); } catch (Exception e) { e.printStackTrace(); System.exit(0); }
         if (line==null) break;
      
         if (!line.startsWith("#"))
         {  parseLine(line);
         }
      } 
      in.close();
   }


   /** Writes Cconfiguration to the given Writer. */
   private void writeTo(Writer wr) throws java.io.IOException
   {  BufferedWriter out=new BufferedWriter(wr);
      out.write(configurable.toString());
      out.close();
   }


   /** Parses a single text line. */
   private void parseLine(String line)
   {  String attribute;
      String value;
      int index=line.indexOf("=");
      if (index>0)
      {  attribute=line.substring(0,index).trim();
         value=line.substring(index+1).trim();
      }
      else
      {  attribute=line;
         value="";
      }
      configurable.parseAttribute(attribute,value);
   }

}

