package conet.configure;

import java.util.HashMap;
import java.util.Iterator;



/** Example of use of Configure class for reading configuration from file.
 */
public class ConfigureExample implements Configurable
{
	private static ConfigureExample instance = null;

	// ********************* Configurable attributes *********************
	public HashMap<String, String[]> params = null;


	public ConfigureExample(){
		params = new HashMap<String, String[]>();
	}

	/** Creates a new ConfigureExample */
	public void init(String file)
	{  // load configuration
		(new Configure(this)).load(file);
		// post-load initialization     
		postLoadInitialization();
	}

	public synchronized static ConfigureExample getInstance(){
		if(instance == null){
			instance = new ConfigureExample();
		}
		return instance;
	}

	/** Inits the ConfigureExample. */
	private void postLoadInitialization()
	{   
		Iterator<String> it = params.keySet().iterator();
		while(it.hasNext()){
			String key = it.next();
			String[] param = params.get(key);
			for (int i=0; i<param.length; i++)
			{ 
				if (param[i]!=null && param[i].equals(Configure.NONE))
					param[i]=null; 
			}
		}
	}


	// ************************* Callback methods ************************

	/** From Configurable. Parses an attribute value. */
	public void parseAttribute(String attribute, String value)
	{  
		final char[] DELIM={' ',','};
		Parser par=new Parser(value);
		params.put(attribute, par.getWordArray(DELIM));
		
	}

	/** From Configurable. Writes all parameters. */
	public String toString()
	{ 
		StringBuffer sb=new StringBuffer();
		Iterator<String> it = params.keySet().iterator();
		while(it.hasNext()){
			String key = it.next();
			sb.append(key).append("=");
			String[] param = params.get(key);
			for (int i=0; i<param.length; i++)
			{ 
				sb.append(param[i]).append("; ");
			}
			sb.append('\n');
		}
		return sb.toString();
	}

	@Override
	public String[] getParams(String key) {

		return params.get(key);
	}

}



