import java.io.BufferedWriter;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.Date;

import edu.byu.nlp.util.TrieMap;
import conet.signaling.message.*;
import conet.configure.*;
import conet.util.*;


public class ConetRoutingEngine {

	public Find_request hostdata_freq;    
	public Register hostdata_reg;
	public Unregister hostdata_unreg;
	public Find_response hostdata_resp;
	public DatagramSocket serverSocket;
	private static TrieMap<String[][]> myMap;
	private static Hashloader refresh = null;
	private static Hashremover remove = null;

	public ConetRoutingEngine(int port, String conf_file){
		ConfigureExample conf = ConfigureExample.getInstance();
		conf.init(conf_file);
		initServer(port);
		initDataStructures();
	}
	
	private static void initDataStructures() {
		if ((ConfigureExample.getInstance().getParams("nre_type")[0]).equals("routing_engine")) 
		{                           
			myMap = new TrieMap<String[][]>();       //creates the REGISTRATION TABLE as a PREFIX TREE HashTable
			refresh = new Hashloader();
			remove = new Hashremover();
			refresh.loader(myMap);                   //REGISTRATION TABLE Refreshing with old data
		}
	}
	
	private void initServer(int port){

		try {
			serverSocket = new DatagramSocket(port);
			System.out.println("Socket listening at port " + port);
		} catch (SocketException e1) {
			e1.printStackTrace();
		}
	}

	public static void main(String[] args){
		  PrintWriter fos=null;
		  //DataOutputStream dos = null;
		  long lDateTime;
		try {
			fos = new PrintWriter("lc_nrs_time.txt");
			//dos = new DataOutputStream(fos);
	
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		  
		ConetRoutingEngine conetRE = null;

		if(args.length == 2)
			conetRE = new ConetRoutingEngine(Integer.parseInt(args[0]), args[1]);
		else if(args.length == 0){
			System.out.println("Loading default values...");
			File configuration_file = new File("conetRE.conf");
			if(!configuration_file.exists())
			{
				System.out.println("No conetRE.conf configuration file found in the current directory. Application is exiting!");
				System.exit(1);
			}			
			conetRE = new ConetRoutingEngine(9999, "conetRE.conf");			
		}
		else{
			printUsage();
			System.exit(1);
		}
			
		{                            // conetRE Running on a SERVER;
			System.out.println("Waiting for Register or Find request..."); 
			try{ 
				while(true){

					byte[] buf = new byte[1000];	
					DatagramPacket dgp = new DatagramPacket(buf, buf.length);   
					conetRE.serverSocket.receive(dgp);
					lDateTime = new Date().getTime();
					fos.println(String.valueOf(lDateTime));
					fos.flush();
					if (Thread.activeCount()<500) {
						//System.out.println("Some info: " + dgp.getAddress().getHostAddress() + ":" + dgp.getPort()+"N. thread"+Thread.activeCount());
						Thread t = new Thread(new MessageHandler(dgp, conetRE.serverSocket));
						t.start();
					}
					
				}        
			}		 
			catch(Exception e){
				e.printStackTrace();
			}
		}
	}

	private static void printUsage() {
		System.out.println("Wrong parameters number!!! Insert either no params to load default values or 2 params, socket port and configuration file path");
	}

	/**
	 * @return the refresh
	 */
	public static Hashloader getRefresh() {
		return refresh;
	}

	/**
	 * @param refresh the refresh to set
	 */
	public static void setRefresh(Hashloader refresh) {
		ConetRoutingEngine.refresh = refresh;
	}

	/**
	 * @return the remove
	 */
	public static Hashremover getRemove() {
		return remove;
	}

	/**
	 * @param remove the remove to set
	 */
	public static void setRemove(Hashremover remove) {
		ConetRoutingEngine.remove = remove;
	}

	/**
	 * @return the myMap
	 */
	public static TrieMap<String[][]> getMyMap() {
		return myMap;
	}

	/**
	 * @param myMap the myMap to set
	 */
	public static void setMyMap(TrieMap<String[][]> myMap) {
		ConetRoutingEngine.myMap = myMap;
	}


}









