import java.net.*;
import java.io.*;

import conet.configure.ConfigureExample;
import conet.signaling.ConetSignal;
import conet.signaling.message.Find_request;
import conet.signaling.message.Find_response;
import conet.signaling.message.Register;
import conet.signaling.message.Signal;
import conet.signaling.message.TCP_Response;
import conet.signaling.message.Unregister;
import conet.util.GetIPaddress;
import conet.util.Long_pref;
import conet.util.Register_management;
import conet.util.distance_compare;
import conet.util.ip_pref;
import edu.byu.nlp.util.TrieMap;

public class MessageHandler implements Runnable{
	private DatagramPacket dgp = null;
	private DatagramSocket opened_socket = null;
	private Register hostdata_reg;
	private Find_request hostdata_find_req;
	private Unregister hostdata_unreg;
	private Find_response hostdata_find_resp;
	private TCP_Response hostdata_tcp_message;
	private InetAddress ip_sender;

	
	public MessageHandler(DatagramPacket dgp, DatagramSocket opened_socket){
		this.dgp = dgp;
		this.opened_socket = opened_socket;
	}
	
	
	public void transfer(String data) throws IOException{	
		byte[] sendData = new byte[1024];
		sendData = data.getBytes();
		DatagramPacket sendPacket = new DatagramPacket(sendData,sendData.length,dgp.getAddress(),dgp.getPort());
		opened_socket.send(sendPacket);
		//System.out.println("Sending to: "+dgp.getAddress().getHostAddress() + ":" + dgp.getPort());

	}

	@Override
	public void run() {
		String MType = printAndCollectValues();
		TrieMap<String[][]> systemMap = ConetRoutingEngine.getMyMap();
		
		if (MType.equals("Register")){                            
			if (!systemMap.containsKey(hostdata_reg.contentName)){
				new Register_management().manage(hostdata_reg , systemMap, ip_sender.getHostAddress());
			}
			else{     //existing entries will be overwritten only for smaller distances;
				boolean compare = new distance_compare().compare(systemMap, hostdata_reg);
				if (compare==true){
					System.out.println("content name will be overwritten");
					ConetRoutingEngine.getRemove().hashremove(systemMap, hostdata_reg.contentName);
					new Register_management().manage(hostdata_reg ,systemMap, ip_sender.getHostAddress());
				}
			}
		}             
		else if(MType.equals("Find_request")){                     
			String nre_type = ConfigureExample.getInstance().getParams("nre_type")[0]; 
			if (nre_type.equals("routing_engine")) {
				//System.out.println ("Longest Prefix Match : ");                         
				
				//computes a Name Based Longest Prefix Match to route a request;
				String longest_name = new Long_pref().longest_pref(systemMap, hostdata_find_req.contentName);
				
				//TODO debug
				//String[][] result = systemMap.get(longest_name);
				/*if(result != null)
					for(int i = 0; i < result.length; i++)
						for(int j = 0; j < result[i].length; j++)
							System.out.println("list["+i+"]["+j+"]= " + result[i][j]);
				// TODO debug to remove
				*/	
				Find_response msgFind = new Find_response(longest_name,systemMap.get(longest_name));
				String serializedClass = ConetSignal.serialize(msgFind);
				try {
					transfer(serializedClass);
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			else if (nre_type.equals("border_node")){
				Find_request msgFind_to_NRS = new Find_request(hostdata_find_req.contentName,hostdata_find_req.address);
				String serializedClass_to_NRS = ConetSignal.serialize(msgFind_to_NRS);

				// send the request towards its neighbors 
				String[] nre_peers = ConfigureExample.getInstance().getParams("nre_peers"); 
				for (int i=0; i< nre_peers.length; i++){
					try {
						transfer(serializedClass_to_NRS);
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
			}
		}
		else if(MType.equals("Find_response")){                                     
			if (hostdata_find_resp.values !=null){	
				GetIPaddress tmpip = new GetIPaddress();
				String next_hop;
				try {
					next_hop = new ip_pref().next_hop(tmpip.ip(),hostdata_find_resp.values );
					//System.out.println("The next hop is :"+ next_hop);
				}catch (SocketException e) {
					e.printStackTrace();
				}
			}
			//Find_request msgFind_to_BG = new Find_request(conetRE.hostdata_resp.contentName,conetRE.hostdata_freq.address); //builds the packet containing the Find_Request to the next_hop;
			//String serializedClass_to_BG = ConetSignal.serialize(msgFind_to_BG);
			//client.transfer(next_hop, dgp.getPort(), serializedClass_to_BG );}
			else 
				System.out.println("Content Name not found"); 
		}
		else if(MType.equals("Unregister")){                                       //every requester will have to find the association that matches its IP address 
			ConetRoutingEngine.getRemove().hashremove(systemMap, hostdata_unreg.contentName);
			Unregister msgUnregister = new Unregister(hostdata_unreg.contentName,hostdata_unreg.address);
			String serializedClass = ConetSignal.serialize(msgUnregister);
			String[] nre_parents = ConfigureExample.getInstance().getParams("nre_parents"); 
			for (int j=0; j< nre_parents.length; j++){
				if(nre_parents != null)
					if (!nre_parents.equals(ip_sender.getHostAddress())){
						try {
							transfer(serializedClass);
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
			}
		}           //sends UnRegister message to peers except for the NRS it just had it from;
	}
	
	
	// TODO to refactor a little bit
	private String printAndCollectValues(){
		String stringaLetta = new String(dgp.getData());
		ip_sender = (dgp.getAddress());
		//System.out.println("Ricevuto :" + stringaLetta);
		
		
		String MType = null;
		Signal deserializedclass[] = ConetSignal.deSerialize(stringaLetta);
		//System.out.println((deserializedclass[0]).getClass().getSimpleName());
		//System.out.println((deserializedclass[0]).getContentName());
		
		if (stringaLetta.contains("Register")){
			MType = "Register";
			hostdata_reg = ((Register)deserializedclass[0]);
			//System.out.println(hostdata_reg.address);	
			//System.out.println(hostdata_reg.flag);
			//System.out.println(hostdata_reg.distance);
		} else if(stringaLetta.contains("Find_request")){
			MType="Find_request";
			hostdata_find_req = ((Find_request)deserializedclass[0]);
			//System.out.println(hostdata_find_req.address);
		}else if(stringaLetta.contains("Unregister")){
			MType="Unregister";
			hostdata_unreg = ((Unregister)deserializedclass[0]);
			//System.out.println(hostdata_unreg.address);
		}else if(stringaLetta.contains("Find_response")){
			MType="Find_response";
			hostdata_find_resp = ((Find_response)deserializedclass[0]);
		}else if(stringaLetta.contains("TCP_Response")){
			MType="TCP_Response";
			hostdata_tcp_message = ((TCP_Response)deserializedclass[0]);
			//System.out.println(hostdata_tcp_message.tcp_message);		
		}
		
		//System.out.println("SENDER IP: "+ ip_sender);
		return MType;
	}
}
		
				
			
	
